import "./index.scss";
import moment from "moment";
import PT from "prop-types";
import React, { useState, useEffect, useRef } from "react";

export const DatePicker = ({ className, dateFormat, displayDateFormat, value, onChange }) => {
	const DEFAULT_DATE_FORMAT = "YYYY-MM-DD";
	const SELECT_MODE_DAY = "SELECT_MODE_DAY";
	const SELECT_MODE_YEAR = "SELECT_MODE_YEAR";
	const SELECT_MODE_MONTH = "SELECT_MODE_MONTH";
	const MONTH_LABELS = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
	const WEEK_LABELS = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];

	const [dialogVisible, setDialogVisibility] = useState(false);
	const [confirmedDate, setConfirmedDate] = useState(null);
	const [selectedMonth, setSelectedMonth] = useState(-1);
	const [selectedYear, setSelectedYear] = useState(-1);
	const [selectedDay, setSelectedDay] = useState(-1);
	const [mode, setMode] = useState(SELECT_MODE_DAY);
	const yearContainerRef = useRef();
	const selectedYearRef = useRef();

	useEffect(() => {
		if (value && moment(value).isValid()) {
			const valueMoment = moment(value, dateFormat || DEFAULT_DATE_FORMAT);
			setSelectedDay(valueMoment.date());
			setSelectedMonth(valueMoment.month());
			setSelectedYear(valueMoment.year());
			setConfirmedDate(valueMoment.format(dateFormat || DEFAULT_DATE_FORMAT));
		} else {
			selectCurrentDate();
			setConfirmedDate(moment().format(dateFormat || DEFAULT_DATE_FORMAT));
		}
	}, [value]);

	useEffect(() => {
		if (mode == SELECT_MODE_YEAR) {
			if (yearContainerRef.current && selectedYearRef.current) {
				const parent = yearContainerRef.current;
				const child = selectedYearRef.current;
				const parentRect = parent.getBoundingClientRect();
				const parentViewableArea = {
					height: parent.clientHeight,
					width: parent.clientWidth
				};

				const childRect = child.getBoundingClientRect();
				const isViewable = childRect.top >= parentRect.top && childRect.top <= parentRect.top + parentViewableArea.height;

				if (!isViewable) parent.scrollTop = childRect.top + parent.scrollTop - parentRect.top;
			}
		}
	}, [mode, selectedYear]);

	const getCurrentMonthParams = () => {
		const _moment = moment([selectedYear, selectedMonth, selectedDay]);
		const year = _moment.year();
		const index = _moment.month();
		const numberOfDays = _moment.daysInMonth();
		const startOffset = moment(`${year}-${index + 1}-01`).day();
		const numberOfWeeks = Math.ceil((numberOfDays + startOffset) / 7);

		return { index, numberOfDays, numberOfWeeks, startingDayOffset: startOffset };
	};

	const selectCurrentDate = () => {
		setSelectedDay(moment().date());
		setSelectedMonth(moment().month());
		setSelectedYear(moment().year());
	};

	const interceptSetSelectedDay = day => {
		if (day > 28 && selectedMonth == 1) setSelectedDay(28);
		setSelectedDay(day);
	};

	const interceptSetSelectedMonth = month => {
		if (selectedDay > 28 && month == 1) setSelectedDay(28);
		setSelectedMonth(month);
	};

	const interceptSetSelectedYear = year => {
		if (selectedDay > 28 && selectedMonth == 1) setSelectedDay(28);
		setSelectedYear(year);
	};

	const interceptModeToggleKeyDown = e => {
		e.preventDefault();

		const selectedMoment = moment([selectedYear, selectedMonth, selectedDay]);
		const moveToDate = (offset, unit) => {
			const newMoment = selectedMoment.add(offset, unit);
			setSelectedYear(newMoment.year());
			setSelectedMonth(newMoment.month());
			setSelectedDay(newMoment.date());
		};

		switch (e.keyCode) {
			case 37: // Left arrow
				if (mode == SELECT_MODE_DAY) moveToDate(-1, "day");
				if (mode == SELECT_MODE_MONTH) moveToDate(-1, "month");
				if (mode == SELECT_MODE_YEAR) moveToDate(-1, "year");
				break;
			case 38: // Top arrow
				if (mode == SELECT_MODE_DAY) moveToDate(-7, "day");
				if (mode == SELECT_MODE_MONTH) moveToDate(-4, "month");
				if (mode == SELECT_MODE_YEAR) moveToDate(-4, "year");
				break;
			case 39: // Right arrow
				if (mode == SELECT_MODE_DAY) moveToDate(1, "day");
				if (mode == SELECT_MODE_MONTH) moveToDate(1, "month");
				if (mode == SELECT_MODE_YEAR) moveToDate(1, "year");
				break;
			case 40: // Down arrow
				if (mode == SELECT_MODE_DAY) moveToDate(7, "day");
				if (mode == SELECT_MODE_MONTH) moveToDate(4, "month");
				if (mode == SELECT_MODE_YEAR) moveToDate(4, "year");
				break;
		}
	};

	const openDialog = () => {
		setDialogVisibility(true);
	};

	const closeDialog = e => {
		e.stopPropagation();
		setDialogVisibility(false);
	};

	const confirmDate = e => {
		e.stopPropagation();
		const confirmedDate = moment([selectedYear, selectedMonth, selectedDay]).format(dateFormat || DEFAULT_DATE_FORMAT);
		setConfirmedDate(confirmedDate);
		onChange(confirmedDate);
		setDialogVisibility(false);
	};

	const clearDate = e => {
		e.stopPropagation();
		setConfirmedDate(null);
		onChange(null);
		setDialogVisibility(false);
	};

	const renderGridRow = ({
		columnCount,
		rowIndex,
		cellCount,
		cellOffset,
		cellLabels,
		cellActive,
		cellRef,
		cellClickHandler,
		cellKeyPressHandler
	}) => {
		const gridRow = [];

		for (var i = 0; i < columnCount; i++) {
			const cellIndex = rowIndex * columnCount + i;
			const cellValue = cellIndex - cellOffset + 1;
			const cellLabel = cellLabels ? cellLabels(cellValue) : cellValue;
			const _cellRef = cellRef ? cellRef(cellValue) : null;
			const cellClassNames = ["calendar-cell"];
			const cellKey = `r${rowIndex}-c${i}`;

			const interceptClickHandler = () => cellClickHandler(cellValue);
			const interceptKeyPressHandler = e => {
				if (e.keyCode == 0 || e.keyCode == 32) cellKeyPressHandler(cellValue);
			};

			if (cellIndex >= cellOffset && cellIndex < cellCount + cellOffset) {
				if (cellActive && cellActive(cellValue)) cellClassNames.push("active");

				gridRow.push(
					<span
						key={cellKey}
						ref={_cellRef}
						onClick={interceptClickHandler}
						onKeyPress={interceptKeyPressHandler}
						className={cellClassNames.join(" ")}
					>
						{cellLabel}
					</span>
				);
			} else gridRow.push(<span className="calendar-cell" key={cellKey} />);
		}

		return gridRow;
	};

	const renderGrid = ({ columnCount, rowCount, cellCount, cellOffset, cellLabels, cellActive, cellRef, cellClickHandler, cellKeyPressHandler }) => {
		const grid = [];

		for (var i = 0; i < rowCount; i++) {
			grid.push(
				<div className="calendar-row">
					{renderGridRow({
						columnCount,
						rowIndex: i,
						cellCount,
						cellOffset,
						cellLabels,
						cellActive,
						cellRef,
						cellClickHandler,
						cellKeyPressHandler
					})}
				</div>
			);
		}

		return grid;
	};

	const renderDayLabels = () =>
		renderGrid({
			cellLabels: i => WEEK_LABELS[i - 1],
			columnCount: 7,
			rowCount: 1,
			cellOffset: 0,
			cellCount: 7
		});

	const renderDays = () => {
		const { numberOfWeeks, numberOfDays, startingDayOffset } = getCurrentMonthParams();

		return renderGrid({
			columnCount: 7,
			rowCount: numberOfWeeks,
			cellCount: numberOfDays,
			cellOffset: startingDayOffset,
			cellClickHandler: interceptSetSelectedDay,
			cellKeyPressHandler: interceptSetSelectedDay,
			cellActive: day => day == selectedDay
		});
	};

	const renderMonths = () => {
		const handler = month => interceptSetSelectedMonth(month - 1);

		return renderGrid({
			columnCount: 4,
			rowCount: 3,
			cellCount: 12,
			cellOffset: 0,
			cellClickHandler: handler,
			cellKeyPressHandler: handler,
			cellActive: month => month - 1 == selectedMonth,
			cellLabels: i => MONTH_LABELS[i - 1]
		});
	};

	const renderYears = () => {
		const currentYear = moment().year();
		const resolveYear = year => currentYear - 100 + year - 1;
		const handler = year => interceptSetSelectedYear(resolveYear(year));

		return renderGrid({
			columnCount: 4,
			rowCount: 75,
			cellCount: 300,
			cellOffset: 0,
			cellClickHandler: handler,
			cellKeyPressHandler: handler,
			cellActive: year => resolveYear(year) == selectedYear,
			cellRef: year => (resolveYear(year) == selectedYear ? selectedYearRef : null),
			cellLabels: resolveYear
		});
	};

	return (
		<span className={`date-picker ${className || "form-control"}`} onClick={openDialog} tabIndex={0}>
			<small>
				{confirmedDate
					? moment(confirmedDate, dateFormat || DEFAULT_DATE_FORMAT).format(displayDateFormat || DEFAULT_DATE_FORMAT)
					: "Not selected"}
			</small>

			{dialogVisible && (
				<div className="date-picker-dialog">
					<div className="date-picker-dialog-content">
						<div className="date-picker-dialog-mode-toggle">
							<p
								className={`month-toggle ${mode == SELECT_MODE_MONTH ? "active" : ""}`}
								onClick={() => setMode(SELECT_MODE_MONTH)}
								onFocus={() => setMode(SELECT_MODE_MONTH)}
								onKeyDown={interceptModeToggleKeyDown}
								tabIndex={0}
							>
								{moment([selectedYear, selectedMonth, selectedDay]).format("MMMM")}
							</p>
							<p
								className={`day-toggle ${mode == SELECT_MODE_DAY ? "active" : ""}`}
								onClick={() => setMode(SELECT_MODE_DAY)}
								onFocus={() => setMode(SELECT_MODE_DAY)}
								onKeyDown={interceptModeToggleKeyDown}
								tabIndex={0}
							>
								{moment([selectedYear, selectedMonth, selectedDay]).format("DD")}
							</p>
							<p
								className={`year-toggle ${mode == SELECT_MODE_YEAR ? "active" : ""}`}
								onClick={() => setMode(SELECT_MODE_YEAR)}
								onFocus={() => setMode(SELECT_MODE_YEAR)}
								onKeyDown={interceptModeToggleKeyDown}
								tabIndex={0}
							>
								{moment([selectedYear, selectedMonth, selectedDay]).format("YYYY")}
							</p>
						</div>

						{mode == SELECT_MODE_DAY && (
							<div className="calendar calendar-days">
								{renderDayLabels()}
								<hr />
								{renderDays()}
							</div>
						)}

						{mode == SELECT_MODE_MONTH && <div className="calendar calendar-months">{renderMonths()}</div>}

						{mode == SELECT_MODE_YEAR && (
							<div className="calendar calendar-years" ref={yearContainerRef}>
								{renderYears()}
							</div>
						)}

						<div className="date-picker-dialog-actions">
							<button type="button" className="btn btn-link btn-sm" onClick={closeDialog}>
								Cancel
							</button>
							<button type="button" className="btn btn-link btn-sm" onClick={selectCurrentDate}>
								Today
							</button>
							<button type="button" className="btn btn-link btn-sm" onClick={clearDate}>
								Clear
							</button>
							<button type="button" className="btn btn-link btn-sm" onClick={confirmDate}>
								<strong>OK</strong>
							</button>
						</div>
					</div>
				</div>
			)}
		</span>
	);
};

DatePicker.propTypes = {
	className: PT.string,
	dateFormat: PT.string,
	displayDateFormat: PT.string,
	value: PT.string,
	onChange: PT.func
};
