import "./index.scss";
import PropTypes from "prop-types";
import React, { useState, useRef, useEffect } from "react";

const FileField = ({ onChange, className, maxFileSize = 0, accept = [] }) => {
	const [dragCounter, setDragCounter] = useState(false);
	const [dragging, setDragging] = useState(false);
	const [fileName, setFileName] = useState("");
	const [error, setError] = useState(null);
	const dropAreaRef = useRef();
	const inputRef = useRef();

	useEffect(() => {
		setError(null);
		setDragCounter(0);
		setDragging(false);
		dropAreaRef.current.addEventListener("dragenter", onDragEnter);
		dropAreaRef.current.addEventListener("dragleave", onDragLeave);
		dropAreaRef.current.addEventListener("dragover", onDragOver);
		dropAreaRef.current.addEventListener("drop", onFileDrop);

		return () => {
			dropAreaRef.current.removeEventListener("dragenter", onDragEnter);
			dropAreaRef.current.removeEventListener("dragleave", onDragLeave);
			dropAreaRef.current.removeEventListener("dragover", onDragOver);
			dropAreaRef.current.removeEventListener("drop", onFileDrop);
		};
	}, []);

	const onDragEnter = e => {
		e.preventDefault();
		e.stopPropagation();
		setDragCounter(dragCounter + 1);
		setDragging(e.dataTransfer.items && e.dataTransfer.items.length > 0);
	};

	const onDragLeave = e => {
		e.preventDefault();
		e.stopPropagation();
		setDragCounter(dragCounter - 1);
		setDragging(dragCounter - 1 != 0);
	};

	const onDragOver = e => {
		e.preventDefault();
		e.stopPropagation();
	};

	const onFileDrop = e => {
		e.preventDefault();
		e.stopPropagation();

		if (e.dataTransfer.files && e.dataTransfer.files.length > 0) {
			handleFileDrop(e.dataTransfer.files[0]);
			e.dataTransfer.clearData();
			setDragCounter(0);
		}

		setDragging(false);
	};

	const openFileDialog = e => {
		e.preventDefault();
		inputRef.current.click();
	};

	const resolveReadableFileSize = size => {
		if (size >= 1073741824) return `${size / 1073741824}GB`;
		if (size >= 1048576) return `${size / 1048576}MB`;
		if (size >= 1024) return `${size / 1024}KB`;
		else return `${size} bytes`;
	};

	const handleFileDrop = file => {
		setFileName("");

		const fn = file.name;
		const extension = fn.substring(fn.lastIndexOf(".") + 1, fn.length);
		if (accept.length != 0 && !accept.includes(extension.toLowerCase())) {
			setError(`File extension must match one of the following formats: ${accept.join(", ")}`);
			return;
		}

		if (maxFileSize > 0 && file.size > maxFileSize) {
			setError(`File size must not be greater than ${resolveReadableFileSize(maxFileSize)}`);
			return;
		}

		setError(null);
		setFileName(file.name);
		if (onChange) onChange(file);
	};

	const combinedClassName = `file-field form-control ${className || ""}`;

	return (
		<div className={combinedClassName} ref={dropAreaRef}>
			{dragging && <div className="drop-overlay">Drop here</div>}
			<span className="placeholder">
				Drag file here or <span onClick={openFileDialog}>select</span>
			</span>
			&nbsp; &nbsp;
			<span className={`file-name ${fileName != "" ? "uploaded" : ""}`}>{!fileName ? "No file selected" : fileName}</span>
			{error && <div className="error">{error}</div>}
			<input type="file" onChange={e => handleFileDrop(e.target.files[0])} ref={inputRef} />
		</div>
	);
};

FileField.propTypes = {
	onChange: PropTypes.func,
	className: PropTypes.string,
	maxFileSize: PropTypes.number,
	accept: PropTypes.arrayOf(PropTypes.string)
};

export { FileField };
