import React from "react";
import { connect } from "react-redux";
import { reduxForm, formValueSelector } from "redux-form";

/**
 * A HOC that returns a reduxForm function wrapper for a component.
 * withForm performs basic validation by checking if required values are empty.
 *
 * @param {string} name The name of the form
 * @param {Array} required A list of required field names
 * @param {Array} accessible A list of field names whose values are passed as props to Component. The values can be accessed under the "fieldValues" props
 * @param {*} initialValues An object of initial key value pairs for the form
 * @param {Function} validateSecondary A function to perform secondary validation. It is passed the values and errors object
 * @param {*} Element The component to be wrapped
 */
export const withForm = (params = { name: "form", required: [], accessible: [], initialValues: {}, validateSecondary: null }) => {
	const form = params.name;
	const required = params.required;
	const accessible = params.accessible;
	const validateSecondary = params.validateSecondary;
	const initialValues = params.initialValues;

	const validate = values => {
		const errors = {};
		for (const field of required) if (!values[field] || values[field] == "") errors[field] = "Required";
		if (validateSecondary) validateSecondary(values, errors);
		return errors;
	};

	const selector = formValueSelector(form);
	const getState = state => {
		if (!accessible) return {};
		if (accessible.length == 0) return { fieldValues: {} };
		if (accessible.length == 1) {
			return {
				fieldValues: { [accessible[0]]: selector(state, accessible[0]) }
			};
		}
		return { fieldValues: selector(state, ...accessible) };
	};

	return Element => {
		return connect(getState)(
			reduxForm({
				form,
				validate,
				initialValues
			})(props => <Element {...props} />)
		);
	};
};
