import "./index.scss";
import PropTypes from "prop-types";
import React, { useState } from "react";
import { LoaderButton } from "../loader-button";

/**
 * A form modal for agent invitation. When the form is submitted, the onInvite
 * is called with name, email and role.
 *
 * @param {boolean} isLoading A boolean indicating whether the invite is being sent
 * @param {array} roles A string array containing the possible agent roles
 * @param {function} onInvite Called when the form is submitted. Receives name, email and role
 */
const InviteAgentModal = ({ roles = [], isLoading, onInvite }) => {
	const [name, setName] = useState("");
	const [email, setEmail] = useState("");
	const [role, setRole] = useState(roles[0] || "");
	const [expanded, setExpanded] = useState(false);
	const [error, setError] = useState(false);

	const validate = () => {
		const values = [name, email, role];
		for (const value of values)
			if (value == "") {
				setError(true);
				return false;
			}

		setError(false);
		return true;
	};

	const submit = e => {
		e.preventDefault();
		if (validate()) onInvite(name, email, role);
	};

	return (
		<span className="invite-agent-modal">
			{!expanded && (
				<button type="button" className="toggle-btn btn btn-outline-primary btn-sm" onClick={() => setExpanded(true)}>
					Invite
				</button>
			)}

			{expanded && (
				<div className="invite-agent-modal-dialog">
					<div className="invite-agent-modal-content">
						<button type="button" className="close" onClick={() => setExpanded(false)}>
							&times;
						</button>

						<form onSubmit={submit}>
							<header className="text-center">Invite Agent</header>

							<label>Agent Name</label>
							<input className="form-control" value={name} onChange={e => setName(e.target.value)} />

							<div className="row">
								<div className="col-6">
									<label>Agent Email</label>
									<input type="email" className="form-control" value={email} onChange={e => setEmail(e.target.value)} />
								</div>

								<div className="col-6">
									<label>Role</label>
									<select className="form-control" value={role} onChange={e => setRole(e.target.value)}>
										{roles.map((role, index) => (
											<option key={index} value={role}>
												{role}
											</option>
										))}
									</select>
								</div>
							</div>

							{error && <div className="error-msg">All fields are required</div>}

							<LoaderButton type="submit" className="btn btn-primary btn-sm" isLoading={isLoading}>
								Send Invite
							</LoaderButton>
						</form>
					</div>
				</div>
			)}
		</span>
	);
};

InviteAgentModal.propTypes = {
	roles: PropTypes.arrayOf(PropTypes.string),
	isLoading: PropTypes.bool,
	onInvite: PropTypes.func
};

export { InviteAgentModal };
